@@ -1,7 +1,7 @@
 #  File src/library/base/R/Defunct.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2020 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 #  https://www.R-project.org/Licenses/
 
 .Defunct <- function(new, package=NULL, msg) {
+    fname <- as.character(sys.call(sys.parent())[[1L]])
     if (missing(msg)) {
-        fname <- as.character(sys.call(sys.parent())[[1L]])
 	msg <- gettextf("'%s' is defunct.\n", fname[length(fname)])
 	if(!missing(new))
 	    msg <- c(msg, gettextf("Use '%s' instead.\n", new))
@@ -29,8 +29,11 @@
 		 else gettext("See help(\"Defunct\")"))
     }
     else msg <- as.character(msg)
+    msg <- paste(msg, collapse = "")
 
-    stop(paste(msg, collapse=""), call. = FALSE, domain = NA)
+    if (missing(new)) new <- NULL
+    stop(errorCondition(msg, old = fname, new = new, package = package,
+                        class = "defunctError"))
 }
 
 ## Version <- function() .Defunct("R.Version")
@@ -177,32 +180,33 @@
 ##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
 ## .Export <- function(...)
 ##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
-## .S3method <- function(generic, class, method)
-##     .Defunct(msg = "namespaces should be specified via the 'NAMESPACE' file")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.14.0
 ## Defunct in 2.15.0
-mem.limits <- function(nsize=NA, vsize=NA) .Defunct("gc")
+## Removed for 4.0.0
+## mem.limits <- function(nsize=NA, vsize=NA) .Defunct("gc")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.13.1
 ## Defunct in 2.15.0
-.readRDS <- function(...) .Defunct("readRDS")
-.saveRDS <- function(...) .Defunct("saveRDS")
+## Removed for 4.0.0
+## .readRDS <- function(...) .Defunct("readRDS")
+## .saveRDS <- function(...) .Defunct("saveRDS")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 2.5.0
-## Removed in 2.15.0
+## Defunct and Removed in 2.15.0
 # Sys.putenv <- function(...) .Defunct("Sys.setenv")
 ## </entry>
 
 ## <entry>
 ## Deprecated in 3.0.0
 ## Defunct in 3.1.0
-.find.package <- function(...).Defunct("find.package")
-.path.package <- function(...).Defunct("path.package")
+## Removed for 4.1.0
+## .find.package <- function(...).Defunct("find.package")
+## .path.package <- function(...).Defunct("path.package")
 ## </entry>
