@@ -102,14 +102,26 @@ function(x, by, FUN, ..., simplify = TRUE, drop = TRUE)
 		    ans <- lapply(X = unname(split(e, grp)), FUN = FUN, ...)
                     if(simplify &&
                        length(len <- unique(lengths(ans))) == 1L) {
+                        ## FIXME                        
                         ## this used to lose classes
                         if(len == 1L) {
                             cl <- lapply(ans, oldClass)
                             cl1 <- cl[[1L]]
-			    ans <- unlist(ans, recursive = FALSE, use.names = FALSE)
-                            if (!is.null(cl1) &&
-                                all(vapply(cl, identical, NA, y = cl1)))
-                                class(ans) <- cl1
+			    ans <- if (!is.null(cl1) &&
+                                       all(vapply(cl, identical, NA,
+                                                  y = cl1)) &&
+                                       ## FIXME: unlist() currently
+                                       ## turns a list of factors into a
+                                       ## factor but c() does not ...
+                                       !inherits(ans[[1L]], "factor"))
+                                       do.call(c, ans)
+                                   else
+                                       unlist(ans, recursive = FALSE,
+                                              use.names = FALSE)
+                            ## FIXME
+                            ## if (!is.null(cl1) &&
+                            ##     all(vapply(cl, identical, NA, y = cl1)))
+                            ##     class(ans) <- cl1
                         } else if(len > 1L)
 			    ans <- matrix(unlist(ans, recursive = FALSE, use.names = FALSE),
                                           ncol = len,
